/*
 * Copyright (c) 2000  Yoshihiro Takagi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __EBookEngine_H
#define __EBookEngine_H

#include <Locker.h>
#include <string>

#include "config.h"
#include <eb/eb.h>
#include <eb/error.h>
#include <eb/text.h>

#include "Observable.h"

struct EB_TitleInfo {
	int  index;
	char *title;
};

struct EB_HeadingInfo {
	int         subbook;
	char        *heading;
	EB_Position *content;
};

typedef void (*PFN_add_title)(const EB_TitleInfo *ebti, void *param);
typedef void (*PFN_add_heading)(const EB_HeadingInfo *ebhf, void *param);

class EBookEngine : public Observable {
public:
	EBookEngine();
	~EBookEngine();

	bool Lock();
	void Unlock();

	bool bind(const char *);
	bool setSubBook(int i);
	int getSubBook() const;

	enum search_method {
		word,
		endword,
		exactword
	};

	int enumTitle(PFN_add_title pfn_add_title, void *param);
	void search(const char *word, PFN_add_heading pfn_add_heading, void *param);
	char *copyright();
	char *getContent(const EB_Position *pos, int subbook = -1);

	void setTargetMode(bool);
	bool getTargetMode() const;

	void setSearchMethod(search_method method);
	search_method getSearchMethod() const;

	string getDictionaryPath();

protected:
	bool createSubBookList();
	int  getSubBookCount();
	char *getSubBookTitle2(int i);

private:
	void __search(const char *word, PFN_add_heading pfn_add_heading, void *param);

	string __path;
	EB_Book __book;
	int __sublistcount;
	int __subbook;
	EB_Subbook_Code *__sublist;
	search_method __search_method;
	BLocker __locker;
	bool __all_subbooks;
};

inline bool EBookEngine::Lock()
{
	return __locker.Lock();
}

inline void EBookEngine::Unlock()
{
	__locker.Unlock();
}

#endif	// __EBookEngine_H
