/*
 * Copyright (c) 2000  Yoshihiro Takagi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <algorithm>
#include "Observable.h"
#include "Observer.h"

Observable::Observable() : changed(false)
{
}

void Observable::addObserver(Observer *o)
{
	if (find(obs.begin(), obs.end(), o) == obs.end()) {
		obs.push_back(o);
	}
}

void Observable::deleteObserver(const Observer *o)
{
	vector<Observer *>::iterator i = find(obs.begin(), obs.end(), o);
	if (i != obs.end()) {
		obs.erase(i);
	}
}

void Observable::notifyObservers(void *arg)
{
	if (hasChanged()) {
		vector<Observer *>::reverse_iterator i = obs.rbegin();
		while (i != obs.rend()) {
			(*i++)->update(this, arg);
		}
		clearChanged();
	}
}

void Observable::deleteObservers()
{
	obs.clear();
}

bool Observable::hasChanged() const
{
	return changed;
}

int  Observable::countObservers() const
{
	return obs.size();
}

void Observable::setChanged()
{
	changed = true;
}

void Observable::clearChanged()
{
	changed = false;
}
