/*
 * Copyright (c) 2000  Yoshihiro Takagi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <View.h>
#include <Application.h>
#include <TextControl.h>
#include <Button.h>
#include <Entry.h>
#include <Path.h>
#include <Message.h>

#include <string>

using namespace std;

#include "SelectCdRomDlg.h"
#include "PrivateMsg.h"

class SelectCdRomView : public BView {
public:
	SelectCdRomView(BMessage *msg, BRect frame);
	virtual void AttachedToWindow();
	virtual void MessageReceived(BMessage *msg);
private:
	BTextControl *__text;
	string __path;
};

SelectCdRomView::SelectCdRomView(BMessage *msg, BRect frame) 
	: BView(frame, "", B_FOLLOW_ALL, B_WILL_DRAW)
{
	const char *path;
	if (B_OK == msg->FindString("dictionary_path", &path)) {
		__path = path;
	}
}

#define M_OK	10

const BRect TEXT_RECT   (  10,  15, 280,  25 );
const BRect BUTTON_RECT ( 300,  10, 350,  20 );
const BRect DLG_RECT    ( 100, 100, 470, 145 );

void SelectCdRomView::AttachedToWindow()
{
	const char *title = "CD-ROM:";
	__text = new BTextControl(TEXT_RECT, "", title, __path.c_str(), NULL);
	AddChild(__text);
	__text->SetDivider(StringWidth(title) + 8);

	BButton *button = new BButton(BUTTON_RECT, "", "OK", new BMessage(M_OK));
	AddChild(button);
	button->MakeDefault(true);
	button->SetTarget(this);
}

void SelectCdRomView::MessageReceived(BMessage *msg)
{
	if (msg->WasDropped()) {
		entry_ref ref;
		msg->FindRef("refs", 0, &ref);
		BPath path(&ref);
		__text->SetText(path.Path());
	} else {
		switch (msg->what) {
		case M_OK:
			if (strcmp(__text->Text(), __path.c_str())) {
				BMessage *m = new BMessage(M_REPLY_SELECT_CDROM);
				m->AddString(DICTIONARY_PATH, __text->Text());
				be_app->PostMessage(m);
			}
			Window()->Quit();
			break;
		default:
			BView::MessageReceived(msg);	
		}
	}
}

SelectCdRomDlg::SelectCdRomDlg(BMessage *msg)
	: BWindow(DLG_RECT, "Select a CD-ROM", 
		B_FLOATING_WINDOW_LOOK,
//		B_NORMAL_WINDOW_FEEL,
		B_MODAL_APP_WINDOW_FEEL,
		B_NOT_CLOSABLE | B_NOT_RESIZABLE | B_NOT_MINIMIZABLE | B_NOT_ZOOMABLE)
{
	SelectCdRomView *view = new SelectCdRomView(msg, Bounds());
	AddChild(view);
}

bool SelectCdRomDlg::QuitRequested()
{
//	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}
